@@ -281,40 +281,11 @@ class Moodle(object):
 
         return headcommit
 
-    def initPHPUnit(self, force=False):
+    def initPHPUnit(self, force=False, prefix=None):
         """Initialise the PHPUnit environment"""
+        raise Exception('This method is deprecated, use phpunit.PHPUnit.init() instead.')
 
-        if self.branch_compare(23, '<'):
-            raise Exception('PHPUnit is only available from Moodle 2.3')
-
-        # Set PHPUnit data root
-        phpunit_dataroot = self.get('dataroot') + '_phpu'
-        self.updateConfig('phpunit_dataroot', phpunit_dataroot)
-        if not os.path.isdir(phpunit_dataroot):
-            mkdir(phpunit_dataroot, 0777)
-
-        # Set PHPUnit prefix
-        phpunit_prefix = 'phpu_'
-        self.updateConfig('phpunit_prefix', phpunit_prefix)
-
-        result = (None, None, None)
-        exception = None
-        try:
-            if force:
-                result = self.cli('/admin/tool/phpunit/cli/util.php', args='--drop', stdout=None, stderr=None)
-            result = self.cli('/admin/tool/phpunit/cli/init.php', stdout=None, stderr=None)
-        except Exception as exception:
-            pass
-
-        if exception != None or result[0] > 0:
-            if result[0] == 129:
-                raise Exception('PHPUnit is not installed on your system')
-            elif result[0] > 0:
-                raise Exception('Something wrong with PHPUnit configuration')
-            else:
-                raise exception
-
-    def initBehat(self, switchcompletely=False):
+    def initBehat(self, switchcompletely=False, force=False, prefix=None, faildumppath=None):
         """Initialise the Behat environment"""
 
         if self.branch_compare(25, '<'):
@@ -330,11 +301,25 @@ class Moodle(object):
         self.updateConfig('behat_dataroot', behat_dataroot)
 
         # Set Behat DB prefix
-        behat_prefix = 'zbehat_'
-        self.updateConfig('behat_prefix', behat_prefix)
+        currentPrefix = self.get('behat_prefix')
+        behat_prefix = prefix or 'zbehat_'
+
+        # Set behat_faildump_path
+        currentFailDumpPath = self.get('behat_faildump_path')
+        if faildumppath and currentFailDumpPath != faildumppath:
+            self.updateConfig('behat_faildump_path', faildumppath)
+        else:
+            self.removeConfig('behat_faildump_path')
+
+        if not currentPrefix or force:
+            self.updateConfig('behat_prefix', behat_prefix)
+        elif currentPrefix != behat_prefix and self.get('dbtype') != 'oci':
+            # Warn that a prefix is already set and we did not change it.
+            # No warning for Oracle as we need to set it to something else.
+            logging.warning('Behat prefix not changed, already set to \'%s\', expected \'%s\'.' % (currentPrefix, behat_prefix))
 
         # Switch completely?
-        if self.branch_compare(27, '<'):
+        if self.branch_compare(26, '<'):
             if switchcompletely:
                 self.updateConfig('behat_switchcompletely', switchcompletely)
                 self.updateConfig('behat_wwwroot', self.get('wwwroot'))
@@ -347,11 +332,21 @@ class Moodle(object):
             if C.get('path') != '' and C.get('path') != None:
                 wwwroot = wwwroot + C.get('path') + '/'
             wwwroot = wwwroot + self.identifier
-            self.updateConfig('behat_wwwroot', wwwroot)
+            currentWwwroot = self.get('behat_wwwroot')
+            if not currentWwwroot or force:
+                self.updateConfig('behat_wwwroot', wwwroot)
+            elif currentWwwroot != wwwroot:
+                logging.warning('Behat wwwroot not changed, already set to \'%s\', expected \'%s\'.' % (currentWwwroot, wwwroot))
 
         # Force a cache purge
         self.purge()
 
+        # Force dropping the tables if there are any.
+        if force:
+            result = self.cli('admin/tool/behat/cli/util.php', args='--drop', stdout=None, stderr=None)
+            if result[0] != 0:
+                raise Exception('Error while initialising Behat. Please try manually.')
+
         # Run the init script.
         result = self.cli('admin/tool/behat/cli/init.php', stdout=None, stderr=None)
         if result[0] != 0:
@@ -374,12 +369,14 @@ class Moodle(object):
             info[k] = v
         return info
 
-    def install(self, dbname=None, engine=None, dataDir=None, fullname=None, dropDb=False):
+    def install(self, dbname=None, engine=None, dataDir=None, fullname=None, dropDb=False, wwwroot=None):
         """Launch the install script of an Instance"""
 
         if self.isInstalled():
             raise InstallException('Instance already installed!')
 
+        if not wwwroot:
+            raise InstallException('Cannot install without a value for wwwroot')
         if dataDir == None or not os.path.isdir(dataDir):
             raise InstallException('Cannot install instance without knowing where the data directory is')
         if dbname == None:
@@ -406,12 +403,6 @@ class Moodle(object):
             db.createdb(dbname)
         db.selectdb(dbname)
 
-        # Defining wwwroot.
-        wwwroot = '%s://%s/' % (C.get('scheme'), C.get('host'))
-        if C.get('path') != '' and C.get('path') != None:
-            wwwroot = wwwroot + C.get('path') + '/'
-        wwwroot = wwwroot + self.identifier
-
         logging.info('Installing %s...' % self.identifier)
         cli = 'admin/cli/install.php'
         params = (wwwroot, dataDir, engine, dbname, C.get('db.%s.user' % engine), C.get('db.%s.passwd' % engine), C.get('db.%s.host' % engine), fullname, self.identifier, C.get('login'), C.get('passwd'))
@@ -427,9 +418,19 @@ class Moodle(object):
                 self.addConfig('sessioncookiepath', '/%s/%s/' % (C.get('path'), self.identifier))
             else:
                 self.addConfig('sessioncookiepath', '/%s/' % self.identifier)
-        except InstallException:
+        except Exception:
             logging.warning('Could not append $CFG->sessioncookiepath to config.php')
 
+        # Add forced $CFG to the config.php if some are globally defined.
+        forceCfg = C.get('forceCfg')
+        if isinstance(forceCfg, dict):
+            for cfgKey, cfgValue in forceCfg.iteritems():
+                try:
+                    logging.info('Setting up forced $CFG->%s to \'%s\' in config.php', cfgKey, cfgValue)
+                    self.addConfig(cfgKey, cfgValue)
+                except Exception:
+                    logging.warning('Could not append $CFG->%s to config.php', cfgKey)
+
         self.reload()
 
     def isInstalled(self):
